# Microeconometrics
# CLASS 5
# Discrete choice models in Apollo (Part 2)
# Wiktor Budzinski, Marek Giergiczny

rm(list=ls())
library(apollo)
library(readxl) 
library(tidyr)
library(matlib)

setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2023\\Class 4") 
data_long <- read_excel("Fishmode.xlsx")

# Transform long format data to short format

database <- pivot_wider(data = data_long, id_cols = id, names_from = alt, 
                        values_from = c("ans","price", "crate"))
database$choice <- database$ans_1 + 2*database$ans_2 + 3*database$ans_3 + 4*database$ans_4 # creating choice variable
database$inc <- data_long$inc[data_long$alt == 1]

modelOutput_settings=list()
modelOutput_settings[['printPVal']] <- 2 # Print p-values
modelOutput_settings[['printClassical']] <- FALSE

################################################# Exercise 1 ########################################
# Estimate a nested multinomial logit 
# 4.1 Fishing from land vs. fishing from boat

apollo_initialise()

apollo_control = list(modelName = "NL1", modelDescr = "Fishmode Nested logit, land vs. boat", indivID = "id",  outputDirectory = "output")

apollo_beta=c(asc_pier = 0, asc_private = 0, asc_charter = 0, bcrate = 0.1, bprice = -0.1, lambda_land = 0.1, lambda_boat = 0.1)
apollo_fixed = c()
apollo_beta=apollo_readBeta(apollo_beta,apollo_fixed,"MNL1",overwriteFixed=FALSE) # using previous model for starting values

apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[["beach"]]  =  bcrate*crate_1 + bprice*price_1
  V[["pier"]]  = asc_pier + bcrate*crate_2 + bprice*price_2
  V[["private"]]  = asc_private + bcrate*crate_3 + bprice*price_3
  V[["charter"]] = asc_charter + bcrate*crate_4 + bprice*price_4
  
  ### Specify nests for NL model
  nlNests      = list(root=1, land= lambda_land, boat = lambda_boat)
  
  ### Specify tree structure for NL model
  nlStructure= list()
  nlStructure[["root"]]   = c("land","boat")
  nlStructure[["land"]]     = c("beach","pier")
  nlStructure[["boat"]]     = c("private","charter")
  
  ### Define settings for NL model
  nl_settings <- list(
    alternatives  = c(beach=1, pier=2, private=3, charter=4), 
    avail         = list(beach=1, pier=1, private=1, charter = 1), 
    choiceVar     = choice,
    utilities     = V,
    nlNests      = nlNests,
    nlStructure  = nlStructure
  )
  
  ### Compute probabilities using NL model
  P[["model"]] = apollo_nl(nl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

nl1 = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(nl1, modelOutput_settings)

# mnl1$maximum -1230.78
nl1$maximum

# 4.1b Land vs. boats separately

apollo_control = list(modelName = "NL1b", modelDescr = "Fishmode Nested logit, land vs. boat", indivID = "id",  outputDirectory = "output")

apollo_beta=c(asc_pier = 0, asc_private = 0, asc_charter = 0, bcrate = 0.1, bprice = -0.1, lambda_land = 0.1)
apollo_fixed = c()
apollo_beta=apollo_readBeta(apollo_beta,apollo_fixed,"MNL1",overwriteFixed=FALSE) # using previous model for starting values

apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[["beach"]]  =  bcrate*crate_1 + bprice*price_1
  V[["pier"]]  = asc_pier + bcrate*crate_2 + bprice*price_2
  V[["private"]]  = asc_private + bcrate*crate_3 + bprice*price_3
  V[["charter"]] = asc_charter + bcrate*crate_4 + bprice*price_4
  
  ### Specify nests for NL model
  nlNests      = list(root=1, land= lambda_land)
  
  ### Specify tree structure for NL model
  nlStructure= list()
  nlStructure[["root"]]   = c("land", "private", "charter")
  nlStructure[["land"]]     = c("beach","pier")
  
  ### Define settings for NL model
  nl_settings <- list(
    alternatives  = c(beach=1, pier=2, private=3, charter=4), 
    avail         = list(beach=1, pier=1, private=1, charter = 1), 
    choiceVar     = choice,
    utilities     = V,
    nlNests      = nlNests,
    nlStructure  = nlStructure
  )
  
  ### Compute probabilities using NL model
  P[["model"]] = apollo_nl(nl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

nl1b = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(nl1b, modelOutput_settings)
# mnl1$maximum -1230.78
nl1b$maximum

# 4.2 Fishing with private resources vs. fishing with chartered boat

apollo_control = list(modelName = "NL2", modelDescr = "Fishmode Nested logit, private vs. chartered", indivID = "id",  outputDirectory = "output")

apollo_beta=c(asc_pier = 0, asc_private = 0, asc_charter = 0, bcrate = 0.1, bprice = -0.1, lambda = 0.1)
apollo_fixed = c()
apollo_beta=apollo_readBeta(apollo_beta,apollo_fixed,"MNL1",overwriteFixed=FALSE) # using previous model for starting values

apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[["beach"]]  =  bcrate*crate_1 + bprice*price_1
  V[["pier"]]  = asc_pier + bcrate*crate_2 + bprice*price_2
  V[["private"]]  = asc_private + bcrate*crate_3 + bprice*price_3
  V[["charter"]] = asc_charter + bcrate*crate_4 + bprice*price_4
  
  ### Specify nests for NL model
  nlNests      = list(root=1, priv = lambda)
  
  ### Specify tree structure for NL model
  nlStructure= list()
  nlStructure[["root"]]   = c("priv","charter")
  nlStructure[["priv"]]     = c("beach","pier", "private")
  
  ### Define settings for NL model
  nl_settings <- list(
    alternatives  = c(beach=1, pier=2, private=3, charter=4), 
    avail         = list(beach=1, pier=1, private=1, charter = 1), 
    choiceVar     = choice,
    utilities     = V,
    nlNests      = nlNests,
    nlStructure  = nlStructure
  )
  
  ### Compute probabilities using NL model
  P[["model"]] = apollo_nl(nl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

nl2 = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(nl2, modelOutput_settings)

setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2023\\Class 4\\output") 
mnl1<-readRDS("MNL1_model.rds")
apollo_lrTest(mnl1, nl2) # LR test

# 4.3 elasticity again

predictions_base = apollo_prediction(nl2, apollo_probabilities, apollo_inputs)
database$price_4 = 1.01*database$price_4
apollo_inputs = apollo_validateInputs()
predictions_new = apollo_prediction(nl2, apollo_probabilities, apollo_inputs)
database$price_4 = (1/1.01)*database$price_4
apollo_inputs = apollo_validateInputs()

### Compute change in probabilities (elasticity)
change <- (predictions_new-predictions_base)/predictions_base
change <- change/0.01
### Get rid of not needed columns
change=change[,-ncol(change)]
change=change[,-c(1,2)]
### Look at mean elasticity
colMeans(change)

# What about changing the price of fishing from beach? 

predictions_base = apollo_prediction(nl2, apollo_probabilities, apollo_inputs)
database$price_1 = 1.01*database$price_1
apollo_inputs = apollo_validateInputs()
predictions_new = apollo_prediction(nl2, apollo_probabilities, apollo_inputs)
database$price_1 = (1/1.01)*database$price_1
apollo_inputs = apollo_validateInputs()

change <- (predictions_new-predictions_base)/predictions_base
change <- change/0.01
change=change[,-ncol(change)]
change=change[,-c(1,2)]
colMeans(change)


################################################# Exercise 2 ########################################
# Random regret

rm(apollo_inputs)
apollo_control = list(modelName  ="RRM",modelDescr ="RRM for fishmode data",
                      indivID    ="id",  outputDirectory = "output")

apollo_beta=c(asc_pier = 0, asc_private = 0, asc_charter = 0, bcrate = 0.1, bprice = -0.1)
apollo_fixed = c()
apollo_inputs = apollo_validateInputs()
# We can add elements to input as needed
apollo_inputs$altnames <- c("beach", "pier", "private", "charter")
apollo_inputs$crate <- data.frame(database$crate_1, database$crate_2, database$crate_3, database$crate_4)
apollo_inputs$price <- data.frame(database$price_1, database$price_2, database$price_3, database$price_4)

# Define RRM model
apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[["beach"]]  =  0
  V[["pier"]]  = asc_pier
  V[["private"]]  = asc_private
  V[["charter"]] = asc_charter
  
  for (i in 1:4) {
    crate_i = apollo_inputs$crate[,i]
    price_i = apollo_inputs$price[,i]
    crate_j = apollo_inputs$crate[,-i]
    price_j = apollo_inputs$price[,-i]
    for (j in 1:3) {
      V[[apollo_inputs$altnames[i]]] = V[[apollo_inputs$altnames[i]]] - log(1+exp(bcrate*(crate_j[,j]-crate_i))) - log(1+exp(bprice*(price_j[,j]-price_i))) 
    }
  }
  
  ### Define settings for MNL model component
  
  mnl_settings = list(
    alternatives  = c(beach=1, pier=2, private=3, charter=4), 
    avail         = list(beach=1, pier=1, private=1, charter = 1), 
    choiceVar     = choice,
    utilities     = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}
# Estimation                                      
rrm <- apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs) 
apollo_modelOutput(rrm, modelOutput_settings)

# not nested model
apollo_basTest(mnl1,rrm)

# Elasticity
predictions_base = apollo_prediction(rrm, apollo_probabilities, apollo_inputs)
database$price_4 = 1.01*database$price_4
apollo_inputs = apollo_validateInputs()
apollo_inputs$altnames <- c("beach", "pier", "private", "charter")
apollo_inputs$crate <- data.frame(database$crate_1, database$crate_2, database$crate_3, database$crate_4)
apollo_inputs$price <- data.frame(database$price_1, database$price_2, database$price_3, database$price_4)

predictions_new = apollo_prediction(rrm, apollo_probabilities, apollo_inputs)
database$price_4 = (1/1.01)*database$price_4
apollo_inputs = apollo_validateInputs()


change <- (predictions_new-predictions_base)/predictions_base
change <- change/0.01
change=change[,-ncol(change)]
change=change[,-c(1,2)]
colMeans(change)