# Microeconometrics
# CLASS 5b
# Endogeneity
# Wiktor Budzinski, Marek Giergiczny

rm(list=ls())
# ctrl+l

setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2023\\Class 5")         # sets the access path (sometimes change the characters '\' to '/' is needed)

# Installing required packages
packages = c("psych", "AER", "stargazer", "Hmisc", "car", "lfe", "quantreg")
find.package(packages)
length(packages)

# if the library is not installed on the given computer, delete the first '#'
# install.packages("psych")
# install.packages("AER")
# install.packages("stargazer")
# install.packages("Hmisc")
# install.packages("car")
# install.packages("lfe")
rm(packages)

# Loading libraries
library(psych)
library(AER)
library(stargazer)
library(Hmisc)
library(car)
library(lfe)
# library(ivpack)
library(ivmodel)
library(REndo)
library(lava)
library(readxl)
library(statmod)
library(boot)

################################################# Exercise 1 ########################################
# Use data from me.twins.rds to estimate a regression explaining how having more than two children affects women's income

twins  <-  readRDS("me.twins.rds")

summary(twins)
describe(twins)
str(twins)

#  Perform an simple linear regression 
Eq1 <- linc ~ morekids + agem + agefstm + blackm + hispm + othracem
model1 <- lm(Eq1, data=twins)
summary(model1)

# Compare the results with the 2SLS estimator (Two Stages Least Squares), where the 'morekid' variable is endogenous, and the 'twins' variable is used as the instrument.

help(ivreg) # direct instrumental-variable estimation (when the number of instruments is equal to the number of predictors)

# 2SLS model -> command ivreg -> after '|' we put instruments we want to use (using '+')
Eq2 <- linc ~ morekids + agem + agefstm + blackm + hispm + othracem | twins + agem + agefstm + blackm + hispm + othracem
model2 <- ivreg(Eq2, data=twins)
summary(model2)

stargazer(model1, model2, type="text", df=FALSE)  

# Check if the instruments are good predictors of the 'morekids' variable
Eq2b <- morekids ~ twins + agem + agefstm + blackm + hispm + othracem
model3 <- lm(Eq2b, data=twins)
summary(model3)

# Compare the results with the model in which the variable 'samesex' is an instrument
Eq3 <- linc ~ morekids + agem + agefstm + blackm + hispm + othracem | samesex + agem + agefstm + blackm + hispm + othracem
model4 <- ivreg(Eq3, data=twins)
summary(model4)

stargazer(model1, model2, model4, type="text", df=FALSE)

Eq3b <- morekids ~ samesex + agem + agefstm + blackm + hispm + othracem
model5 <- lm(Eq3b, data=twins)
summary(model5)

################################################# Exercise 2 ########################################
# Test the model for endogeneity

# Hausman test
# We estimate both OLS and 2SLS
# Sequence matters! (model2 - model1)
coef_diff =  coef(model2) - coef(model1)
vc_diff   =  vcov(model2) - vcov(model1)
endo_diff =  as.vector(t(coef_diff) %*% solve(vc_diff) %*% coef_diff)
pchisq(endo_diff, df=6, lower.tail = FALSE)                # df=6 because there is 6 variables (morekids, agem, agefstm, blackm, hispm, othracem)
# H0: difference in coefficients are not systematic
# p-value = 6.037858e-13

# Wu-Hausman test
summary(model2, diagnostics=TRUE)

# We repeat tests, but this time we are using 'samesex' and 'twins' variables as in instrument
Eq3c <- linc ~ morekids + agem + agefstm + blackm + hispm + othracem | twins + samesex + agem + agefstm + blackm + hispm + othracem
model4b <- ivreg(Eq3c, data=twins)

# Wu-Hausman test and Sargan tests
summary(model4b, diagnostics=TRUE)

model4c <- lm(model4b$residuals ~ twins$twins + twins$samesex)
summary(model4c)
6.378e-06*394840 # N*R^2 - Sargan

# Hausman test

coef_diff1 = coef(model4) - coef(model1)
vc_diff1 = vcov(model4) - vcov(model1)
endo_diff1 = as.vector(t(coef_diff1) %*% solve(vc_diff1) %*% coef_diff1)
pchisq(endo_diff1, df=6, lower.tail = FALSE)               # df=6 because there is 6 variables (morekids, agem, agefstm, blackm, hispm, othracem)
# H0: difference in coefficients are not systematic
# p-value = 0.2858484

summary(model4, diagnostics=TRUE)

################################################# Exercise 3 ########################################
# Weak instruments

cor(twins, method="pearson")

variables=cbind(twins$morekids, twins$samesex, twins$twins)
colnames(variables)=c("morekids","samesex","twins")
cor(variables, method="pearson")                           # kendall oraz spearman correlation can be computed also
cor(variables)                                             # pearson's correlation is default

# Regression in which we explain endogenous variable with instruments (and with exogenous variables)
Eq4 <- morekids ~ samesex + twins + agem + agefstm + blackm + hispm + othracem
model6 <- lm(Eq4, data=twins)
summary(model6)

# total significance of the instruments
linearHypothesis(model=model6, c("samesex=0", "twins=0"), data=twins)

# Check the weakness of the instruments

# 'twins'
summary(model2, diagnostics=TRUE)

# Conditional F statistic for weak instruments in an IV-estimation with multiple endogenous variables (model needs to have a class "felm")
#Eq5 <- linc ~ agem + agefstm + blackm + hispm + othracem | 0 | (morekids ~ twins + agem + agefstm + blackm + hispm + othracem)
Eq5 <- linc ~ agem + agefstm + blackm + hispm + othracem | 0 | (morekids ~ twins)
model7 <- felm(Eq5, data=twins)
summary(model7)
condfstat(model7)
# 'twins' + 'samesex'

# Conditional F statistic for weak instruments in an IV-estimation with multiple endogenous variables (model needs to have a class "felm")
# Eq6 <- linc ~ agem + agefstm + blackm + hispm + othracem | 0 | (morekids ~ twins + samesex + agem + agefstm + blackm + hispm + othracem)
Eq6 <- linc ~ agem + agefstm + blackm + hispm + othracem | 0 | (morekids ~ twins + samesex)
model8 <- felm(Eq6, data=twins)
summary(model8)
condfstat(model8)

# Anderson Rubin  confidence intervals

# model2b <- ivreg(Eq2, data=twins, x = TRUE) # x = TRUE needs to be added
# summary(model2b)
# anderson.rubin.ci(model2b)
# -0.727640-1.96*0.126141;  -0.727640+1.96*0.126141

# Alternatively
model2c <- ivmodelFormula(Eq2, data=twins)
summary(model2c)

################################################# Exercise 4 ########################################
# Use of proxies and multiple indicators solution

rm(list=ls())
data <- read_excel("nls80.xls")
Eq1 <- lwage ~ educ + exper + tenure + married + south + urban + black
model1 <- lm(Eq1, data = data)
summary(model1)

# Proxy
Eq2 <- lwage ~ educ + exper + tenure + married + south + urban + black + iq
model2 <- lm(Eq2, data = data)
summary(model2)

# MIS

Eq3 <- lwage ~ educ + exper + tenure + married + south + urban + black + iq| kww + educ + exper + tenure + married + south + urban + black
model3 <- ivreg(Eq3, data=data)
summary(model3)


Eq3b <- lwage ~ educ + exper + tenure + married + south + urban + black + kww| iq + educ + exper + tenure + married + south + urban + black
model3b <- ivreg(Eq3b, data=data)
summary(model3b)

stargazer(model2, model3,model3b, type="text", df=FALSE)  

# compare it with regular iv method
Eq4 <- lwage ~ educ + exper + tenure + married + south + urban + black| meduc + feduc  + exper + tenure + married + south + urban + black
model4 <- ivreg(Eq4, data=data)
summary(model4, diagnostics=TRUE)

stargazer(model2, model3,model3b, model4, type="text", df=FALSE)  

# Is parents education correlated with ability
cor(cbind(data$feduc, data$meduc, data$iq), use = "complete.obs")     

# Use of SEM instead of MIS

m1 <- lvm()
data$liq <- log(data$iq)
data$lkww <- log(data$kww)

regression(m1) <- c(lwage, liq, lkww) ~ u1
regression(m1) <- lwage ~ educ + exper + tenure + married + south + urban + black
latent(m1) <- ~u1
covariance(m1, u1 ~ u1) <- 1
intercept(m1, ~u1) <- 0
summary(m1)

model5<-estimate(m1, data = data)
summary(model5)

# Maybe u1 is correlated with other factors

regression(m1) <- u1 ~ exper + tenure + married + south + urban + black
model5b<-estimate(m1, data = data)
summary(model5b)