
# Loading libraries
library(psych)
library(AER)
library(stargazer)
library(Hmisc)
library(car)
library(lfe)
library(REndo)
library(ivmodel)

## Simulation Example no 1.

set.seed(1234567) # setting seed

# Strong instrument

x1 <- runif(1000,-2,2)
xmiss <- rnorm(1000,0,1)
z <- runif(1000,-2,2) # iv
eps <- rnorm(1000,0,2)
eps2 <- rnorm(1000,0,1)
alpha <- 1
xe <- 1*xmiss + alpha*z + eps2
y<- x1 + xe - xmiss + eps

m1 <-lm(y~ xe+x1)
summary(m1)

m2 <-ivreg(y~ xe + x1 | z + x1)
summary(m2)

m2b <- felm(y~ x1 | 0 | (xe ~ z))
summary(m2b)
condfstat(m2b)
m2c <- ivmodelFormula(y~ xe + x1 | z + x1)
summary(m2c)

# Weak instrument

alpha <- 0.05
xe <- 1*xmiss + alpha*z + eps2
y<- x1 + xe - xmiss + eps

m1 <-lm(y~ xe+x1)
summary(m1)

m2 <-ivreg(y~ xe + x1 | z + x1, x = TRUE)
summary(m2)

m2b <- felm(y~ x1 | 0 | (xe ~ z))
summary(m2b)
condfstat(m2b)

m2c <- ivmodelFormula(y~ xe + x1 | z + x1)
summary(m2c)

## Simulation Example no 2.
# Latent instrumental variable

rm(list=ls())
set.seed(1234567) # setting seed

xmiss <- rnorm(1000,0,1)
z <- runif(1000,0,1) > 0.6 # iv (unobserved)
eps <- rnorm(1000,0,2)
eps2 <- rnorm(1000,0,1)
alpha <- 2
xe <- 1*xmiss + alpha*z + eps2
y<- 0.5+1.5*xe - xmiss + eps

m1 <-ivreg(y~ xe | z)
summary(m1)

data <- cbind.data.frame(y,xe)
m2 <- latentIV(y~ xe, data = data)
summary(m2)

# pi1 represents the estimated group mean of the first group.
# pi2 The estimated group mean of the second group of the instrumental variables Z.
# theta5 The probability of being in the first group of the instruments.
# theta6 The variance of eps
# theta7 The covariance
# theta8 The variance of eps2


## Simulation Example no 3.
# Copula solution

rm(list=ls())
set.seed(1234567) # setting seed

x1 <- runif(1000,-2,2)
xmiss <- rnorm(1000,0,1)
z <- runif(1000,-2,2) # iv
eps <- rnorm(1000,0,2)
eps2 <- rnorm(1000,0,1)
alpha <- 1
xe <- 5+ 2*xmiss + alpha*z + exp(eps2)
y<- x1 + xe - xmiss + eps
hist(xe)

m0 <- lm(y~ xe + x1)
summary(m0)

m1 <-ivreg(y~ xe + x1 | z + x1)
summary(m1)

data <- cbind.data.frame(y,xe,x1)
m2 <- copulaCorrection(y~ xe + x1 | continuous(xe), data = data, num.boots = 200)
summary(m2)
